@@ -22,12 +22,13 @@
  */
 
 #include "./gxtuner.h"
-#include "./config.h"
+#include "./cmdparser.h"
 
 #define P_(s) (s)   // FIXME -> gettext
 
 enum {
     PROP_FREQ = 1,
+    PROP_REFERENCE_PITCH = 2,
 };
 
 static gboolean gtk_tuner_expose (GtkWidget *widget, GdkEventExpose *event);
@@ -41,26 +42,25 @@ static void gx_tuner_get_property(
     GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
 static const int tuner_width = 100;
-static const int tuner_height = 90;
+static const int tuner_height = 70;
 static const double rect_width = 100;
 static const double rect_height = 60;
 
 static const double dashes[] = {
-    0.0,                    /* ink  */
-    rect_height,            /* skip */
-    10.0,                    /* ink  */
-    10.0                    /* skip */
+    0.0,                      /* ink  */
+    rect_height,              /* skip */
+    10.0,                     /* ink  */
+    10.0                      /* skip */
 };
 
 static const double dash_ind[] = {
-    0,                        /* ink  */
-    5,                        /* skip */
+    0,                         /* ink  */
+    5,                         /* skip */
     rect_height-20,            /* ink  */
-    100.0                    /* skip */
+    100.0                      /* skip */
 };
 
-GType gx_tuner_get_type(void)
-{
+GType gx_tuner_get_type(void) {
     static GType tuner_type = 0;
 
     if (!tuner_type) {
@@ -82,8 +82,7 @@ GType gx_tuner_get_type(void)
     return tuner_type;
 }
 
-static void gx_tuner_class_init(GxTunerClass *klass)
-{
+static void gx_tuner_class_init(GxTunerClass *klass) {
     GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
     GTK_WIDGET_CLASS(klass)->expose_event = gtk_tuner_expose;
     gobject_class->set_property = gx_tuner_set_property;
@@ -93,48 +92,66 @@ static void gx_tuner_class_init(GxTunerClass *klass)
             "freq", P_("Frequency"),
             P_("The frequency for which tuning is displayed"),
             0.0, 1000.0, 0.0, GParamFlags(GTK_PARAM_READWRITE)));
+    g_object_class_install_property(
+        gobject_class, PROP_REFERENCE_PITCH, g_param_spec_double (
+            "reference-pitch", P_("Reference Pitch"),
+            P_("The frequency for which tuning is displayed"),
+            400.0, 500.0, 440.0, GParamFlags(GTK_PARAM_READWRITE)));
     klass->surface_tuner = cairo_image_surface_create(
         CAIRO_FORMAT_ARGB32, tuner_width*3., tuner_height*3.);
     g_assert(klass->surface_tuner != NULL);
     draw_background(klass->surface_tuner);
 }
 
-static void gx_tuner_base_class_finalize(GxTunerClass *klass)
-{
+static void gx_tuner_base_class_finalize(GxTunerClass *klass) {
     if (klass->surface_tuner) {
         g_object_unref(klass->surface_tuner);
     }
 }
 
-static void gx_tuner_init (GxTuner *tuner)
-{
+static void gx_tuner_init (GxTuner *tuner) {
+    g_assert(GX_IS_TUNER(tuner));
+    tuner->freq = 0;
+    tuner->reference_pitch = 440.0;
     GtkWidget *widget = GTK_WIDGET(tuner);
     widget->requisition.width = tuner_width;
     widget->requisition.height = tuner_height;
 }
 
-void gx_tuner_set_freq(GxTuner *tuner, double freq)
-{
+void gx_tuner_set_freq(GxTuner *tuner, double freq) {
     g_assert(GX_IS_TUNER(tuner));
     tuner->freq = freq;
     gtk_widget_queue_draw(GTK_WIDGET(tuner));
     g_object_notify(G_OBJECT(tuner), "freq");
 }
 
-GtkWidget *gx_tuner_new(void)
-{
+void gx_tuner_set_reference_pitch(GxTuner *tuner, double reference_pitch) {
+    g_assert(GX_IS_TUNER(tuner));
+    tuner->reference_pitch = reference_pitch;
+    gtk_widget_queue_draw(GTK_WIDGET(tuner));
+    g_object_notify(G_OBJECT(tuner), "reference-pitch");
+}
+
+double gx_tuner_get_reference_pitch(GxTuner *tuner) {
+    g_assert(GX_IS_TUNER(tuner));
+    return tuner->reference_pitch;
+}
+
+GtkWidget *gx_tuner_new(void) {
     return (GtkWidget*)g_object_new(GX_TYPE_TUNER, NULL);
 }
 
 static void gx_tuner_set_property(GObject *object, guint prop_id,
-                                      const GValue *value, GParamSpec *pspec)
-{
+                                      const GValue *value, GParamSpec *pspec) {
     GxTuner *tuner = GX_TUNER(object);
 
     switch(prop_id) {
     case PROP_FREQ:
         gx_tuner_set_freq(tuner, g_value_get_double(value));
         break;
+    case PROP_REFERENCE_PITCH:
+        gx_tuner_set_reference_pitch(tuner, g_value_get_double(value));
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -142,22 +159,23 @@ static void gx_tuner_set_property(GObject *object, guint prop_id,
 }
 
 static void gx_tuner_get_property(GObject *object, guint prop_id,
-                                      GValue *value, GParamSpec *pspec)
-{
+                                      GValue *value, GParamSpec *pspec) {
     GxTuner *tuner = GX_TUNER(object);
 
     switch(prop_id) {
     case PROP_FREQ:
         g_value_set_double(value, tuner->freq);
         break;
+    case PROP_REFERENCE_PITCH:
+        g_value_set_double(value, tuner->reference_pitch);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
     }
 }
 
-static gboolean gtk_tuner_expose (GtkWidget *widget, GdkEventExpose *event)
-{
+static gboolean gtk_tuner_expose (GtkWidget *widget, GdkEventExpose *event) {
     static const char* note[12] = {"A ","A#","B ","C ","C#","D ","D#","E ","F ","F#","G ","G#"};
     static const char* octave[9] = {"0","1","2","3","4","4","6","7"," "};
     static int indicate_oc = 0;
@@ -165,7 +183,7 @@ static gboolean gtk_tuner_expose (GtkWidget *widget, GdkEventExpose *event)
     cairo_t *cr;
 
     double x0      = (widget->allocation.width - 100) * 0.5;
-    double y0      = (widget->allocation.height - 90) * 0.5;
+    double y0      = (widget->allocation.height - 70) * 0.5;
 
     cr = gdk_cairo_create(widget->window);
     cairo_save(cr);
@@ -174,12 +192,12 @@ static gboolean gtk_tuner_expose (GtkWidget *widget, GdkEventExpose *event)
     static double grow   = 0.;
 
     if(widget->allocation.width > widget->allocation.height +(10.*grow*3)) {
-        grow = (widget->allocation.height/90.)/10.;
+        grow = (widget->allocation.height/70.)/10.;
     } else {
         grow =  (widget->allocation.width/100.)/10.;
     }
     
-    grow_h = (widget->allocation.height/90.)/3.;
+    grow_h = (widget->allocation.height/70.)/3.;
     grow_w =  (widget->allocation.width/100.)/3.;
     
     cairo_translate(cr, -x0*grow_w, -y0*grow_h);
@@ -195,7 +213,7 @@ static gboolean gtk_tuner_expose (GtkWidget *widget, GdkEventExpose *event)
     float scale = -0.5;
     if (tuner->freq) {
         float freq_is = tuner->freq;
-        float fvis = 12 * log2f(freq_is/440.0);
+        float fvis = 12 * log2f(freq_is/tuner->reference_pitch);
         int vis = int(round(fvis));
         scale = (fvis-vis) / 2;
         vis = vis % 12;
@@ -260,8 +278,7 @@ static gboolean gtk_tuner_expose (GtkWidget *widget, GdkEventExpose *event)
     cairo_set_dash (cr, dash_ind, sizeof(dash_ind)/sizeof(dash_ind[0]), 0);
     cairo_line_to(cr, (scale*2*rect_width)+x0+47, y0+(scale*scale*30)+2);
     cairo_set_source_rgb(cr,  0.5, 0.1, 0.1);
-    cairo_stroke(cr);
-    
+    cairo_stroke(cr);   
     cairo_destroy(cr);
 
     return FALSE;
@@ -270,8 +287,7 @@ static gboolean gtk_tuner_expose (GtkWidget *widget, GdkEventExpose *event)
 /*
 ** paint tuner background picture (the non-changing parts)
 */
-static void draw_background(cairo_surface_t *surface)
-{
+static void draw_background(cairo_surface_t *surface) {
     cairo_t *cr;
 
     double x0      = 0;
@@ -355,10 +371,6 @@ namespace gx_engine {
 const int DOWNSAMPLE = 16;
 // Number of times that the FFT is zero-padded to increase frequency resolution.
 const int ZERO_PADDING_FACTOR = 64;
-// Value of the threshold above which the processing is activated.
-const float    SIGNAL_THRESHOLD_ON = 0.001;
-// Value of the threshold below which the input audio signal is deactivated.
-const float SIGNAL_THRESHOLD_OFF = 0.0009;
 // Time between frequency estimates (in seconds)
 const float TRACKER_PERIOD = 0.1;
 
@@ -367,11 +379,24 @@ void *PitchTracker::static_run(void *p) {
     return NULL;
 }
 
+void PitchTracker::set_threshold(float t) {
+    // Value of the threshold above which the processing is activated.
+    SIGNAL_THRESHOLD_ON = t;
+    // Value of the threshold below which the input audio signal is deactivated.
+    SIGNAL_THRESHOLD_OFF = t-0.0001;
+}
+
+float PitchTracker::get_threshold() {
+    // Value of the threshold above which the processing is activated.
+    return SIGNAL_THRESHOLD_ON;
+}
+
 PitchTracker::PitchTracker()
     : error(false),
     busy(false),
     tick(0),
     m_pthr(0),
+    resamp(new Resampler),
     m_buffer(new float[MAX_FFT_SIZE]),
     m_bufferIndex(0),
     m_audioLevel(false),
@@ -394,7 +419,6 @@ PitchTracker::PitchTracker()
     }
 }
 
-
 PitchTracker::~PitchTracker() {
     fftwf_destroy_plan(m_fftwPlanFFT);
     fftwf_destroy_plan(m_fftwPlanIFFT);
@@ -403,7 +427,6 @@ PitchTracker::~PitchTracker() {
     delete[] m_buffer;
 }
 
-
 bool PitchTracker::setParameters(int sampleRate, int fftSize) {
     assert(fftSize <= MAX_FFT_SIZE);
 
@@ -411,7 +434,9 @@ bool PitchTracker::setParameters(int sampleRate, int fftSize) {
         return false;
     }
     m_sampleRate = sampleRate / DOWNSAMPLE;
-    resamp.setup(sampleRate, m_sampleRate, 1, 16); // 16 == least quality
+    resamp->setup(sampleRate, m_sampleRate, 1, 16); // 16 == least quality
+    SIGNAL_THRESHOLD_ON = 0.001;
+    SIGNAL_THRESHOLD_OFF = 0.0009;
 
     if (m_fftSize != fftSize) {
         m_fftSize = fftSize;
@@ -436,8 +461,16 @@ bool PitchTracker::setParameters(int sampleRate, int fftSize) {
     return !error;
 }
 
+void PitchTracker::stop_thread() {
+    pthread_cancel (m_pthr);
+    pthread_join (m_pthr, NULL);
+    sem_post(&m_trig);
+    delete resamp;
+    resamp = 0;
+}
+
 void PitchTracker::start_thread() {
-    int                min, max;
+    int                min = 0, max = 0;
     pthread_attr_t     attr;
     struct sched_param  spar;
     int priority, policy;
@@ -450,7 +483,8 @@ void PitchTracker::start_thread() {
     if (priority < min) priority = min;
     spar.sched_priority = priority;
     pthread_attr_init(&attr);
-    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+    pthread_attr_setdetachstate(&attr,PTHREAD_CREATE_JOINABLE );
+    pthread_setcancelstate (PTHREAD_CANCEL_ENABLE, NULL);
     pthread_attr_setschedpolicy(&attr, policy);
     pthread_attr_setschedparam(&attr, &spar);
     pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);
@@ -507,19 +541,19 @@ void PitchTracker::add(int count, float* input) {
     if (error) {
         return;
     }
-    resamp.inp_count = count;
-    resamp.inp_data = input;
+    resamp->inp_count = count;
+    resamp->inp_data = input;
     for (;;) {
-        resamp.out_data = &m_buffer[m_bufferIndex];
+        resamp->out_data = &m_buffer[m_bufferIndex];
         int n = MAX_FFT_SIZE - m_bufferIndex;
-        resamp.out_count = n;
-        resamp.process();
-        n -= resamp.out_count; // n := number of output samples
+        resamp->out_count = n;
+        resamp->process();
+        n -= resamp->out_count; // n := number of output samples
         if (!n) { // all soaked up by filter
             return;
         }
         m_bufferIndex = (m_bufferIndex + n) % MAX_FFT_SIZE;
-        if (resamp.inp_count == 0) {
+        if (resamp->inp_count == 0) {
             break;
         }
     }
@@ -549,6 +583,7 @@ void PitchTracker::run() {
     for (;;) {
         busy = false;
         sem_wait(&m_trig);
+        pthread_testcancel();
         busy = true;
         if (error) {
             continue;
@@ -601,39 +636,40 @@ void PitchTracker::run() {
 }
 
 void PitchTracker::setEstimatedFrequency(float freq) {
-    gx_gui::fConsta4 = freq;
+    gx_gui::tw.fConsta4 = freq;
 }
 
 PitchTracker pitch_tracker;
-}
+} // end namespace gx_engine
 
-static void destroy( GtkWidget *widget, gpointer   data ) {
-    jack_port_unregister(gx_jack::client, gx_jack::input_port);
-    jack_deactivate(gx_jack::client);
-    jack_client_close(gx_jack::client);
-    if (gx_gui::g_threads > 0) {
-        g_source_remove(gx_gui::g_threads);
-    }
-    gtk_main_quit ();
-}
 
-static gboolean delete_event(GtkWidget *widget, GdkEvent  *event,
-                             gpointer   data ) {
-    return FALSE;
-}
 
 namespace gx_jack {
 
 static bool gx_jack_init() {
     client_name =   "gxtuner";
     input_port  =   0;
-
+#ifdef HAVE_JACK_SESSION
+    if (!cmd.get_optvar(JACK_UUID).empty()) {
+        client = jack_client_open (client_name.c_str(), jack_options_t(
+                 JackNoStartServer | JackSessionID), &jackstat,
+                 cmd.get_optvar(JACK_UUID).c_str());
+    } else {
+        client = jack_client_open(client_name.c_str(), JackNoStartServer, &jackstat);
+    }
+#else
     client = jack_client_open(client_name.c_str(), JackNoStartServer, &jackstat);
+#endif
     if (client) {
         jack_sr = jack_get_sample_rate(client); // jack sample rate
         jack_bs = jack_get_buffer_size(client); // jack buffer size
-        jack_set_process_callback(client, gx_jack_process, 0);
-        jack_on_shutdown (client, jack_shutdown, 0);
+        jack_set_process_callback(client, gx_jack_process, 0); // compute
+        jack_on_shutdown (client, jack_shutdown, 0);  // shutdown clean up
+#ifdef HAVE_JACK_SESSION
+        if (jack_set_session_callback) {
+            jack_set_session_callback(client, gx_jack_session_callback, 0);
+        }
+#endif
         input_port = jack_port_register(client, "in_0", JACK_DEFAULT_AUDIO_TYPE,
                      JackPortIsInput|JackPortIsTerminal, 0);
     } else {
@@ -641,14 +677,29 @@ static bool gx_jack_init() {
         exit(1);
     }
 
-    if (jack_activate (client)) {
-        fprintf (stderr, "cannot activate client\n");
-        exit (1);
-    }
     return true;
 }
 
-void jack_shutdown (void *arg) {exit (1);}
+static void gx_jack_activate() {
+    if (jack_activate (client)) {
+        fprintf (stderr, "cannot activate client\n");
+        exit (1);
+    } else {
+#ifdef HAVE_JACK_SESSION
+        if (!cmd.get_optvar(JACK_INP).empty() && cmd.get_optvar(JACK_UUID).empty()) {
+            jack_connect(client, cmd.get_optvar(JACK_INP).c_str(),
+                         jack_port_name(input_port));
+        }
+#else
+        if (!cmd.get_optvar(JACK_INP).empty()) {
+            jack_connect(client, cmd.get_optvar(JACK_INP).c_str(),
+                         jack_port_name(input_port));
+        }
+#endif
+    }
+}
+
+void jack_shutdown (void *arg) {gtk_main_quit ();}
 
 static int gx_jack_process(jack_nframes_t nframes, void *arg) {
     float *input = static_cast<float *>
@@ -656,36 +707,230 @@ static int gx_jack_process(jack_nframes_t nframes, void *arg) {
     gx_engine::pitch_tracker.add(nframes, input);
     return 0;
 }
+
+#ifdef HAVE_JACK_SESSION
+static int gx_jack_session_callback_helper(gpointer data) {
+    jack_session_event_t *event = static_cast<jack_session_event_t *>(data);
+    gint x,y,w,l;
+    gtk_window_get_position(GTK_WINDOW(gx_gui::tw.window), &x, &y);
+    gtk_window_get_size(GTK_WINDOW(gx_gui::tw.window), &w, &l);
+    double p = gx_tuner_get_reference_pitch(GX_TUNER(gx_gui::tw.tuner));
+    double t = gx_engine::pitch_tracker.get_threshold();
+    std::string cmd("gxtuner -U ");
+    cmd += event->client_uuid;
+    char buffer [100];
+    sprintf (buffer, " -x %i -y %i -w %i -l %i -p %f -t %f ",x, y, w, l, p, t);
+    cmd += buffer;
+    event->command_line = strdup(cmd.c_str());
+
+    jack_session_reply(client, event);
+
+    if (event->type == JackSessionSaveAndQuit) {
+        jack_port_unregister(client, input_port);
+        jack_deactivate(client);
+        jack_client_close(client);
+        if (gx_gui::tw.g_threads > 0) {
+            g_source_remove(gx_gui::tw.g_threads);
+        }
+        gtk_main_quit ();
+        gx_engine::pitch_tracker.stop_thread();
+        jack_session_event_free(event);
+        exit(0);
+    }
+    jack_session_event_free(event);
+    return 0;
 }
 
-static gboolean gx_update_all_gui(gpointer arg) {
-    gx_tuner_set_freq(GX_TUNER(gx_gui::tuner),gx_gui::fConsta4);
+void gx_jack_session_callback(jack_session_event_t *event, void *arg) {
+    gtk_idle_add(gx_jack_session_callback_helper, static_cast<void *>(event));
+}
+#endif
+} // end namespace gx_jack
+
+namespace gx_gui {
+
+static void destroy( GtkWidget *widget, gpointer data) {
+    jack_port_unregister(gx_jack::client, gx_jack::input_port);
+    jack_deactivate(gx_jack::client);
+    jack_client_close(gx_jack::client);
+    if (tw.g_threads > 0) {
+        g_source_remove(tw.g_threads);
+    }
+    gtk_main_quit ();
+}
+
+static gboolean delete_event(GtkWidget *widget, GdkEvent *event,
+                             gpointer data) {
+    return FALSE;
+}
+
+static gboolean gx_update_frequency(gpointer arg) {
+    gx_tuner_set_freq(GX_TUNER(tw.tuner),tw.fConsta4);
     return true;
 }
 
-int main(int argc, char *argv[]) {
-    gx_gui::g_threads = 0;
-    g_thread_init(NULL);
-    gx_jack::gx_jack_init();
-    gx_engine::pitch_tracker.init();
-    gtk_init (&argc, &argv);
-    GError* err = NULL;
-    GtkWidget *window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_default_size(GTK_WINDOW(window), 120,100);
+static gboolean ref_freq_changed(gpointer arg) {
+    gx_tuner_set_reference_pitch(GX_TUNER(tw.tuner),
+        gtk_adjustment_get_value(GTK_ADJUSTMENT(arg)));
+    return true;
+}
+
+static gboolean threshold_changed(gpointer arg) {
+    gx_engine::pitch_tracker.set_threshold(
+        gtk_adjustment_get_value(GTK_ADJUSTMENT(arg)));
+    return true;
+}
+
+static void signal_handler(int sig) {
+    // print out a warning
+    g_print ("signal: %i received, exiting ...\n", sig);
+    destroy(NULL,NULL);
+}
+
+TunerWidget::TunerWidget() {}
+TunerWidget::~TunerWidget() {}
+
+void TunerWidget::create_window() {
+    // create main window and set icon to use
+    err = NULL;
+    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
     gtk_window_set_icon_from_file(GTK_WINDOW(window),
         (std::string(PIXMAPS_DIR) + "/gxtuner.png").c_str(),&err);
     if (err != NULL) g_error_free(err);
-    gx_gui::tuner = gx_tuner_new();
-    gtk_container_add (GTK_CONTAINER (window), gx_gui::tuner);
-
+    // create all used widgets
+    tuner = gx_tuner_new();
+    box = gtk_vbox_new(false, 0);
+    hbox = gtk_hbox_new(false, 0);
+    abox = gtk_alignment_new (1.0,1.0,0.0,0.0);
+    bbox = gtk_alignment_new (0.0,0.0,0.0,0.0);
+    adj = gtk_adjustment_new(440, 427, 453, 0.1, 1.0, 0);
+    spinner = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.1, 1);
+    adjt = gtk_adjustment_new(0.001, 0.001, 0.5, 0.001, 1.0, 0);
+    spinnert = gtk_spin_button_new(GTK_ADJUSTMENT(adjt), 0.001, 3);
+    // set some options to widgets
+    gtk_widget_set_can_focus(GTK_WIDGET(spinner), false);
+    gtk_widget_set_can_focus(GTK_WIDGET(spinnert), false);
+    gtk_widget_set_can_default(GTK_WIDGET(spinner), true);
+    gtk_widget_set_can_default(GTK_WIDGET(spinnert), true);
+    gtk_widget_set_tooltip_text(GTK_WIDGET(spinner),"reference pitch");
+    gtk_widget_set_tooltip_text(GTK_WIDGET(spinnert),"threshold");
+    // set some colour for widgets
+    GdkColor colorOwn;
+    gdk_color_parse("#000000", &colorOwn);
+    gtk_widget_modify_bg(window, GTK_STATE_NORMAL, &colorOwn);
+    gtk_widget_modify_bg(spinner, GTK_STATE_NORMAL, &colorOwn);
+    gtk_widget_modify_bg(spinnert, GTK_STATE_NORMAL, &colorOwn);
+    gtk_widget_modify_base(spinner, GTK_STATE_NORMAL, &colorOwn);
+    gtk_widget_modify_base(spinnert, GTK_STATE_NORMAL, &colorOwn);
+    gdk_color_parse("#292929", &colorOwn);
+    gtk_widget_modify_fg(spinner, GTK_STATE_NORMAL, &colorOwn);
+    gtk_widget_modify_fg(spinnert, GTK_STATE_NORMAL, &colorOwn);
+    gdk_color_parse("#797979", &colorOwn);
+    gtk_widget_modify_fg(spinner, GTK_STATE_PRELIGHT, &colorOwn);
+    gtk_widget_modify_fg(spinnert, GTK_STATE_PRELIGHT, &colorOwn);
+    gdk_color_parse("#999909", &colorOwn);
+    gtk_widget_modify_text(spinner, GTK_STATE_NORMAL, &colorOwn);
+    gtk_widget_modify_text(spinnert, GTK_STATE_NORMAL, &colorOwn);
+    // stack all together
+    gtk_container_add (GTK_CONTAINER(window), box);
+    gtk_box_pack_start(GTK_BOX(box), tuner,true,true,0);
+    gtk_box_pack_end(GTK_BOX(box), hbox, false,false,0);
+    gtk_container_add (GTK_CONTAINER (bbox), spinnert);
+    gtk_box_pack_start(GTK_BOX(hbox),bbox,false,false,0);
+    gtk_box_pack_end(GTK_BOX(hbox),abox,false,false,0);
+    gtk_container_add (GTK_CONTAINER (abox), spinner);
+    // connect the signal handlers 
+    g_signal_connect(GTK_OBJECT(adj), "value-changed",
+        G_CALLBACK(ref_freq_changed),(gpointer)adj);
+    g_signal_connect(GTK_OBJECT(adjt), "value-changed",
+        G_CALLBACK(threshold_changed),(gpointer)adjt);
     g_signal_connect (window, "delete-event",
             G_CALLBACK (delete_event), NULL);
     g_signal_connect (window, "destroy",
             G_CALLBACK (destroy), NULL);
-    gx_tuner_set_freq(GX_TUNER(gx_gui::tuner),gx_gui::fConsta4);
-    gtk_widget_show_all  (window);
-    gx_gui::g_threads = g_timeout_add(100, gx_update_all_gui, 0);
+}
+
+void TunerWidget::parse_cmd() {
+    // *** commandline parsing ***
+    // set default window size optional by command line options
+    int x = 120;
+    int y = 100;        
+    if (!cmd.get_optvar(SIZE_X).empty()) {
+        x = atoi(cmd.get_optvar(SIZE_X).c_str());
+    }
+    if (!cmd.get_optvar(SIZE_Y).empty()) {
+        y = atoi(cmd.get_optvar(SIZE_Y).c_str());
+    }
+    gtk_window_set_default_size(GTK_WINDOW(window), x,y);
+    // set default window position optional by command line options
+    if (!cmd.get_optvar(POS_X).empty()) {
+        x = atoi(cmd.get_optvar(POS_X).c_str());
+    } else {
+        x = gdk_screen_width()/2 -x;
+    }
+    if (!cmd.get_optvar(POS_Y).empty()) {
+        y = atoi(cmd.get_optvar(POS_Y).c_str());
+    } else {
+        y = gdk_screen_height()/2 -y;
+    }
+    gtk_window_move(GTK_WINDOW(window),x,y);
+    // set reference pitch and threshold by command line options
+    double p,t;
+    if (!cmd.get_optvar(PITCH).empty()) {
+        p = atof(cmd.get_optvar(PITCH).c_str());
+    } else {
+        p = 440.0;
+    }
+    if (!cmd.get_optvar(THRESHOLD).empty()) {
+        t = atof(cmd.get_optvar(THRESHOLD).c_str());
+    } else {
+        t = 0.001;
+    }
+    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj),p);
+    gtk_adjustment_set_value(GTK_ADJUSTMENT(adjt),t);
+}
+
+void TunerWidget::show() {
+   // finaly show the window with all widgets
+    gtk_widget_show_all(window);
+}
+TunerWidget tw;
+} // end namespace gx_gui
+
+int main(int argc, char *argv[]) {
+
+    // trap signals to quit clean
+    signal(SIGTERM, gx_gui::signal_handler);
+    signal(SIGHUP,  gx_gui::signal_handler);
+    signal(SIGINT,  gx_gui::signal_handler);
+    signal(SIGQUIT, gx_gui::signal_handler);
+    // init thread system
+    gx_gui::tw.g_threads = 0;
+    g_thread_init(NULL);
+    // process comandline options
+    cmd.process_cmdline_options(argc, argv);
+    // init jack
+    gx_jack::gx_jack_init();
+    // init gtk
+    gtk_init (&argc, &argv);
+    // activate jack
+    gx_jack::gx_jack_activate();
+    // start pitchtracker
+    gx_engine::pitch_tracker.init();
+    // create window
+    gx_gui::tw.create_window();
+    gx_gui::tw.parse_cmd();
+    gx_gui::tw.show();
+    // start thread to update the frequency
+    gx_gui::tw.g_threads = g_timeout_add(
+        100, gx_gui::gx_update_frequency, 0);
+    // run main programm
     gtk_main ();
 
+    gx_engine::pitch_tracker.stop_thread();
+
     return 0;
 }
+
+
+
